package com.ebuilder.ebaftersales.business.hello;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.jboss.ejb3.annotation.Management;
import org.jboss.naming.Util;


/**
 * Copyright 2010, eBuilder AB
 * All rights reserved
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE
 * of eBuilder AB;
 * the contents of this file may not be disclosed to third parties,
 * copied or duplicated in any form, in whole or in part,
 * without the prior written permission of eBuilder AB.
 *
 * Author: Lasantha Ranaweera
 * Dec 3, 20123:55:27 PM
 */

@Management(HelloService.class)
public class HelloServiceBean implements HelloService {

	private boolean masterNode = false;
	private HelloService reference;
	private static final String JNDI_NAME = "custom/HelloServiceBean";
	
	private static final String JNP_PARTITION_NAME = "jnp.partitionName";
    private static final String NAMING_CONTEXT_FACTORY = "org.jnp.interfaces.NamingContextFactory";
    private static final String NAMING_INTERFACES = "jboss.naming:org.jnp.interfaces";
    private static final String DEFAULT_PARTITION_NAME = "DefaultPartition";
    
    private static List<ExternalConnection> connectionList = new ArrayList<ExternalConnection>();
    private static final Long MAX_POOL_SIZE = new Long(30);
	
	/**
	 * Called by HASingletonController on singleton start
	 */
	public void startSingleton() throws NamingException {
		System.out.println("Starting Singleton Service !!!!");
		
		Properties contextProperties = new Properties();
		contextProperties.put(Context.INITIAL_CONTEXT_FACTORY, NAMING_CONTEXT_FACTORY);
		contextProperties.put(Context.URL_PKG_PREFIXES, NAMING_INTERFACES);
		contextProperties.put(JNP_PARTITION_NAME, DEFAULT_PARTITION_NAME);
		
		for(int i=0; i< MAX_POOL_SIZE; i++){
			ExternalConnection connection = new ExternalConnection(new Long(i));
			connectionList.add(connection);
		}
		
		Context ctx = new InitialContext(contextProperties);
        Util.rebind(ctx, JNDI_NAME, reference);
		masterNode = true;
	}

	/**
	 * Called by HASingletonController on singleton stop
	 */
	public void stopSingleton() throws NamingException {
		System.out.println("Stopping singleton service !!!!");
		masterNode = false;
	}

	@Override
	public synchronized ExternalConnection acquireConnection() {
		ExternalConnection connection = null;
		System.out.println("Getting connection from pool size of "+connectionList.size());
		if(connectionList.size() > 0){
			connection = connectionList.get(0);
			connectionList.remove(0);
		}
		return connection;
	}

	@Override
	public synchronized void releaseConnection() {
		if(connectionList.size() < MAX_POOL_SIZE){
			System.out.println("Adding connection in to pool to "+connectionList.size());
			ExternalConnection connection = new ExternalConnection(new Long(0));
			connectionList.add(connection);
		}
	}


}
